function Main()
% tic();
% Parameter settings
lambda          = 4;           % MBS density (per km^2)
lambdaUE        = 100;         % UE density (per km^2)
LenSimArea      = 4.2686;      % Length of the simulation area (in km)
SE_Th           = 0.01;        % SE threshold (in bps/Hz)
P_mac_dBm       = 46;          % Macro Tx power (in dBm)
P_uav_dBm       = 30;          % UAV Tx power (in dBm)
K_dB            = -11;         % Attenuation due to geometrical parameters of antennas (in dB)
PercDestroyMBS  = 90;          % percentage of MBSs to destroy
HeightMBS       = 0.03048; %100 feet     % Height of the MBSs (in km)
HeightUAV1       = 0.12192; %400 feet     % Height of the UAVs (in km)
% HeightUAV1      = 0.1524; %500 feet   % Height of the UAVs (in km)
HeightUAV2      = 0.04572; % 150 feet   % Height of the UAVs (in km)
PlotCovArea     = 1;
PLE             = 4;

Nm = round(lambda*LenSimArea^2); % Number of MBSs
Nue = round(lambdaUE*(0.7*LenSimArea)^2); % Number of UEs
Pm = 10^(P_mac_dBm/10)/1000 * 10^(K_dB/10); % Net transmit power in Watts
Puav = 10.^(P_uav_dBm/10)/1000* 10^(K_dB/10); % Net transmit power in Watts

% Initially generate random MBS and UE location coordinates
% within a given simulation area
Xm = LenSimArea*(rand(Nm,1)-0.5);
Ym = LenSimArea*(rand(Nm,1)-0.5);
Xue = 0.7*LenSimArea*(rand(Nue,1)-0.5);
Yue = 0.7*LenSimArea*(rand(Nue,1)-0.5);

Zm = HeightMBS*ones(size(Xm));
% Calculate and plot throughput coverage area
clc;
CalcCovArea(Xm, Ym, Zm, Pm, PlotCovArea, LenSimArea, PLE, SE_Th, lambdaUE);

NoDestroyMBS = round(PercDestroyMBS/100*Nm);
idx_destroy = zeros(NoDestroyMBS,1);
temp = 1:length(Xm);
for n = 1:NoDestroyMBS
    idx = randi([1,length(temp)]);
    idx_destroy(n) = temp(idx);
    temp(idx) = [];
end

% Destroy the marked MBSs
Xm(idx_destroy) = [];
Ym(idx_destroy) = [];
Zm = HeightMBS*ones(size(Xm));

% Calculate and plot throughput coverage area
CalcCovArea(Xm, Ym, Zm, Pm, PlotCovArea, LenSimArea, PLE, SE_Th, lambdaUE);

% Optimize the UAV location to have maximum 5th percentile SE
% % For 5 UAVs Height 1
NoUAVs = 5;
[~,Xuav, Yuav] = GA_OptimizeUAVlocation(Xm, Ym, Zm, Xue, Yue, Pm, Puav, NoUAVs, HeightUAV1, LenSimArea, PLE);
Zuav = HeightUAV1*ones(size(Xuav));
CalcCovArea(Xm, Ym, Zm, Pm, PlotCovArea, LenSimArea, PLE, SE_Th, lambdaUE, Xuav, Yuav, Zuav, Puav);

NoUAVs = 10;
% % For 10 UAVs Height 1
% Height 1
[~,Xuav, Yuav] = GA_OptimizeUAVlocation(Xm, Ym, Zm, Xue, Yue, Pm, Puav, NoUAVs, HeightUAV1, LenSimArea, PLE);
Zuav = HeightUAV1*ones(size(Xuav));
CalcCovArea(Xm, Ym, Zm, Pm, PlotCovArea, LenSimArea, PLE, SE_Th, lambdaUE, Xuav, Yuav, Zuav, Puav);
% Height 2
[~,Xuav, Yuav] = GA_OptimizeUAVlocation(Xm, Ym, Zm, Xue, Yue, Pm, Puav, NoUAVs, HeightUAV2, LenSimArea, PLE);
Zuav = HeightUAV2*ones(size(Xuav));
CalcCovArea(Xm, Ym, Zm, Pm, PlotCovArea, LenSimArea, PLE, SE_Th, lambdaUE, Xuav, Yuav, Zuav, Puav);
% sure = toc
end