function main_CDF_plots()

REB1_dB      = 0:6:12;
DestroySeq1  = [200, 390];
NoUAVs1      = [16 30 56];
NoIte1       = 6;
SEthreshold  = 10e-3;

plotstyle = {'b','r--','m-.','g','k','y','b:','r:','m:','k:'};

REB1 = 10.^(REB1_dB/10);

idx_strt = zeros(size(NoUAVs1));
idx_strt(1) = 1;
for x = 2:length(NoUAVs1)
    idx_strt(x) = idx_strt(x-1)+NoUAVs1(x-1);
end

idx_stop = zeros(size(NoUAVs1));
idx_stop(1) = NoUAVs1(1);
for x = 2:length(NoUAVs1)
    idx_stop(x) = idx_stop(x-1)+NoUAVs1(x);
end

FifthPercentileSE = zeros(length(REB1_dB),length(NoUAVs1),length(DestroySeq1));
CovProb = zeros(length(REB1_dB),length(NoUAVs1),length(DestroySeq1));
for mm = 1:length(DestroySeq1)
    for kk = 1:length(NoUAVs1)
        idx1 = idx_strt(kk):idx_stop(kk);
        for nn = 1:length(REB1_dB)
            SE = [];
            for ii = 1:NoIte1
                load(['Data_REBdB_' num2str(REB1_dB(nn)) '_DstrSeq_' num2str(DestroySeq1(mm)) '_ite_' num2str(ii) '.mat']);
                Zuav = HeightUAV*ones(size(Xuav));
                SE = [SE; ComputeSE(Xm, Ym, Zm, Xue, Yue, Pm, PLE, Xuav(idx1), Yuav(idx1), Zuav(idx1), Puav, REB1(nn))];
            end

            [SE_CDF, SE_x] = cdfcalc(SE); % Compute CDF
            figure(mm);
            plot(SE_x,SE_CDF(1:end-1),plotstyle{nn}); hold on;

            % Calculate 5th percentile SE
            IdxLower = find(SE_CDF <= 0.05,1,'last');
            IdxHigher = find(SE_CDF >= 0.05,1,'first');
            if((SE_CDF(IdxHigher)-0.05) < (0.05-SE_CDF(IdxLower)))
                FifthPercentileSE(nn,kk,mm) = SE_x(IdxHigher);
            else
                FifthPercentileSE(nn,kk,mm) = SE_x(IdxLower);
            end

            % Calculate the coverage probability
            if(SEthreshold<SE_x(1))
                CovProb(nn,kk,mm) = 1;
            else
                OutageProb = interp1(SE_x,SE_CDF(1:end-1),SEthreshold,'spline');
                CovProb(nn,kk,mm) = 1-OutageProb;
            end
        end
    end
    figure(mm); hold off; grid on;
    xlabel('per-user SE (in dB)');
    ylabel('CDF');
    legend('REB = 0 dB','REB = 6 dB','REB = 12 dB');
end

plotstyle = {'b-','r--','m-.','g:'};
for mm = 1:length(DestroySeq1)
    for kk = 1:length(NoUAVs1)
        figure(200);
        plot(REB1_dB,FifthPercentileSE(:,kk,mm),plotstyle{kk},'LineWidth',2);
        hold on;
        figure(201);
        plot(REB1_dB,CovProb(:,kk,mm),plotstyle{kk},'LineWidth',2);
        hold on;
    end
end
figure(200); hold off; grid on;
xlabel('Range expansion bias (in dB)');
ylabel('5th percentile SE (bps/Hz)');
figure(201); hold off; grid on;
xlabel('Range expansion bias (in dB)');
ylabel('SE coverage probability');

end

function [SE] = ComputeSE(Xm, Ym, Zm, Xue, Yue, Pm, PLE, Xuav, Yuav, Zuav, Puav, REB)

    % Fixed parameters
    Dmin_macro  = 0.035;    % Minimum distance between MBS and UE (in km)
    Dmin_uav    = 0.01;     % Minimum distance between UAV and UE (in km)

    % Coordinates of all the MBS and UAV nodes (the third column
    % indicates whether it is MBS (:1) or UAV (:2) )
    All_Node_Coor = [Xm Ym Zm ones(size(Xm)); Xuav Yuav Zuav 2*ones(size(Xuav))];

    % Find distances between every UE from every MBS and UAV
    [Xue_mat, Xmu_mat] = ndgrid(Xue, All_Node_Coor(:,1));
    [Yue_mat, Ymu_mat] = ndgrid(Yue, All_Node_Coor(:,2));
    [Zue_mat, Zmu_mat] = ndgrid(zeros(size(Xue)), All_Node_Coor(:,3));
    Dmu = sqrt((Xmu_mat-Xue_mat).^2 + (Ymu_mat-Yue_mat).^2 + (Zmu_mat-Zue_mat).^2);

    % Find MOI and UOI for every UE
    [moi_dist, idx_moi] = min(Dmu(:,1:length(Xm)),[],2);
    [uoi_dist, idx_uoi] = min(Dmu(:,length(Xm)+1:end),[],2);
    idx_uoi = idx_uoi + length(Xm);

    % Exclude the UEs which do not satisfy the min. distance contraints
    idx = (moi_dist < Dmin_macro) | (uoi_dist < Dmin_uav);
    Dmu(idx,:) = [];
    idx_moi(idx) = [];
    idx_uoi(idx) = [];
    Xue(idx) = [];

%     % Plot the MBS, UAV and UE locations
%     Yue(idx) = [];
%     figure(3);
%     plot(Xm,Ym,'o'); hold on; plot(Xue,Yue,'.g'); plot(Xuav,Yuav,'dr'); hold off;
%     xlabel('X distance (km)'); ylabel('Y distance (km)');
%     legend('MBS','UE','UAV');

    % Calculte Received powers at all the UEs from every MBS assuming
    % only the large-scale path-loss (No Rayleigh fading)
    Pr = [Pm*ones(length(Xue),length(Xm)) Puav*ones(length(Xue),length(Xuav))]./(Dmu*1e3).^PLE;

    % find received powers from MOI for each UE
    Pr_mac = Pr(sub2ind(size(Pr),(1:length(idx_moi))',idx_moi));
    Pr_uav = Pr(sub2ind(size(Pr),(1:length(idx_uoi))',idx_uoi));

    % Calculate total interference at each UE except MOI/UOI
    Z = sum(Pr,2) - Pr_mac - Pr_uav;

    % Calculte MBS and UAV SIRs
    SIR_mac = Pr_mac./(Z+Pr_uav);
    SIR_uav = Pr_uav./(Z+Pr_mac);
    
    % MBS or UAV cell selection
    idx_mue = SIR_mac > REB*SIR_uav;
    idx_uue = SIR_mac <= REB*SIR_uav;

    % Assuming round-robin scheduling, calculate the SEs of all the UEs
    % MUEs
    idx_moi = idx_moi(idx_mue);
    SIR_mac = SIR_mac(idx_mue);
    moi_list = tabulate([-1; idx_moi]);
    moi_list(1,:) = [];
    N_mue = moi_list(:,2);
    moi_list = moi_list(:,1);
    SE_mue = zeros(size(SIR_mac));
    for x = 1:length(moi_list)
        idx = idx_moi == moi_list(x);
        SE_mue(idx) = log2(1+SIR_mac(idx))/N_mue(x);
    end
    
    % UUEs
    idx_uoi = idx_uoi(idx_uue);
    SIR_uav = SIR_uav(idx_uue);
    uoi_list = tabulate([-1; idx_uoi]);
    uoi_list(1,:) = [];
    N_uue = uoi_list(:,2);
    uoi_list = uoi_list(:,1);
    SE_uue = zeros(size(SIR_uav));
    for x = 1:length(uoi_list)
        idx = idx_uoi == uoi_list(x);
        SE_uue(idx) = log2(1+SIR_uav(idx))/N_uue(x);
    end

    SE = [SE_mue; SE_uue];
%     [SE_CDF, SE_x] = cdfcalc(SE);

%     % Plot CDF of SE
%     figure(3);
%     plot([0;SE_x], SE_CDF,'r');
%     xlabel('SE (bps/Hz)');
%     ylabel('CDF'); grid on;

end