function CovPerc = CalcCovArea(Xm, Ym, Zm, Pm, PlotCovArea, LenSimArea, PLE, SE_Th, lambdaUE, Xuav, Yuav, Zuav, Puav, REB)

Cmue_col = [0 0 1];         % Color for coverage area of Macrocells (RGB)
Cuue_col = [0.8 0.8 0.8];   % Color for coverage area of UAVcells (RGB)
Cotg_col = [1 1 1];         % Color for area of outage (RGB)
Res = 500;                  % Layout grid resolution

b = 0.7*LenSimArea/2;
[Xue, Yue] = meshgrid(linspace(-b,b,Res),linspace(-b,b,Res));
Xue = Xue(:);
Yue = Yue(:);
SE_Th1 = lambdaUE*(2*b/Res)^2*SE_Th;

%%%%%%%%%%%%%%%%%%%%%%%%%%%  Case without UAV   %%%%%%%%%%%%%%%%%%%%%%%%%%%
if(nargin == 9)
    % Coordinates of all the macrocell and picocell nodes (the third column
    % indicates whether it is a eNB (:1) or PeNB (:2))
    All_Node_Coor = [Xm Ym Zm ones(size(Xm))];

    % Find distances between every UE from every BS
    [Xue_mat, Xm_mat] = ndgrid(Xue, All_Node_Coor(:,1));
    [Yue_mat, Ym_mat] = ndgrid(Yue, All_Node_Coor(:,2));
    [Zue_mat, Zm_mat] = ndgrid(zeros(size(Xue)), All_Node_Coor(:,3));
    Dmpu = sqrt((Xm_mat-Xue_mat).^2 + (Ym_mat-Yue_mat).^2 + (Zm_mat-Zue_mat).^2);
    clear Xue_mat Yue_mat Zue_mat Xm_mat Ym_mat Zm_mat
    
    % Find MOI for every UE
    [~,idx_moi] = min(Dmpu,[],2);
    
    % Calculte average Received powers at all the UEs from every BS
%     Pr = Pm./(10^4.64*(Dmpu*1e3).^PLE);
    Pr = Pm./(Dmpu*1e3).^PLE;

    % find received powers from MOI and POI for each UE
    Pr_mac = Pr(sub2ind(size(Pr),(1:length(idx_moi))',idx_moi));

    % Calculate total interference at each UE except MOI
    Z = sum(Pr,2) - Pr_mac;

    % Calculte Macro and pico SINRs
    SIR_mac = Pr_mac./Z;

    % Assuming round-robin scheduling, calculate the SEs of all the UEs
    % MUEs
    moi_list = tabulate([-1; idx_moi]);
    moi_list(1,:) = [];
    N_mue = moi_list(:,2);
    moi_list = moi_list(:,1);
    SE = zeros(size(SIR_mac));
    for x = 1:length(moi_list)
        idx = idx_moi == moi_list(x);
        SE(idx) = log2(1+SIR_mac(idx))/N_mue(x);
    end

    % Calculate the percentage coverage area
    idx_otg = SE < SE_Th1;
    CovPerc = (length(idx_otg)-sum(idx_otg))/length(idx_otg)*100;

    if(PlotCovArea)
        % Plot the coverage layout
        idx_otg = reshape(idx_otg,Res,Res);
        Rmat = Cmue_col(1)*ones(Res,Res);
        Gmat = Cmue_col(2)*ones(Res,Res);
        Bmat = Cmue_col(3)*ones(Res,Res);
        Rmat(idx_otg) = Cotg_col(1); Gmat(idx_otg) = Cotg_col(2); Bmat(idx_otg) = Cotg_col(3);
        Cmat(:,:,1) = Rmat; Cmat(:,:,2) = Gmat; Cmat(:,:,3) = Bmat;
        figure();
        image(linspace(-b,b,Res),linspace(-b,b,Res),Cmat);
        hold on;
        plot(Xm,Ym,'dr','MarkerFaceColor','r')
        [vx,vy] = voronoi(Xm,Ym);
        plot(vx,vy,'k');
        set(gca,'FontSize',14);
        xlim([-b b]);
        xlabel('X-axis distance (Km)','FontSize',14);
        ylabel('Y-axis distance (Km)','FontSize',14);
        title(['SE coverage = ' num2str(CovPerc) '%']);
        ylim([-b b]);
        legend('MBS');
        hold off;
    end
    
%%%%%%%%%%%%%%%%%%%%%%%%%%    Case with UAVs   %%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif(nargin == 14)
    % Coordinates of all the macrocell and picocell nodes (the third column
    % indicates whether it is a eNB (:1) or PeNB (:2))
    All_Node_Coor = [Xm Ym Zm ones(size(Xm)); Xuav Yuav Zuav 3*ones(size(Xuav))];

    % Find distances between every UE from every BS
    [Xue_mat, Xmu_mat] = ndgrid(Xue, All_Node_Coor(:,1));
    [Yue_mat, Ymu_mat] = ndgrid(Yue, All_Node_Coor(:,2));
    [Zue_mat, Zmu_mat] = ndgrid(zeros(size(Xue)), All_Node_Coor(:,3));
    Dmu = sqrt((Xmu_mat-Xue_mat).^2 + (Ymu_mat-Yue_mat).^2 + (Zmu_mat-Zue_mat).^2);
    clear Xue_mat Yue_mat Zue_mat Xmu_mat Ymu_mat Zmu_mat

    % Find MOI and UOI for every UE
    [~,idx_moi] = min(Dmu(:,1:length(Xm)),[],2);
    [~,idx_uoi] = min(Dmu(:,(length(Xm)+1):end),[],2);
    idx_uoi = idx_uoi + length(Xm);
    
    % Calculte average Received powers at all the UEs from every BS
    Pmu_mat = [Pm*ones(length(Xue),length(Xm)) Puav*ones(length(Xue),length(Xuav))];
    Pr = Pmu_mat./(Dmu*1e3).^PLE;

    % find received powers from MOI and UOI for each UE
    Pr_mac = Pr(sub2ind(size(Pr),(1:length(idx_moi))',idx_moi));
    Pr_uav = Pr(sub2ind(size(Pr),(1:length(idx_uoi))',idx_uoi));

    % Calculate total interference at each UE except MOI/POI
    Z = sum(Pr,2) - Pr_mac - Pr_uav;

    % Calculte Macro and pico SINRs
    SIR_mac = Pr_mac./(Pr_uav + Z);
    SIR_uav = Pr_uav./(Pr_mac + Z);

    % Find the MUEs and UUEs
    idx_mue = SIR_mac > REB*SIR_uav;
    idx_uue = SIR_uav >= REB*SIR_mac;

    % Assuming round-robin scheduling, calculate the SEs of all the UEs
    % MUEs
    idx = idx_moi(idx_mue);
    moi_list = tabulate([-1; idx]);
    moi_list(1,:) = [];
    N_mue = moi_list(:,2);
    moi_list = moi_list(:,1);
    SE = zeros(size(idx_mue));
    for x = 1:length(moi_list)
        idx = idx_moi == moi_list(x) & idx_mue;
        SE(idx) = log2(1+SIR_mac(idx))/N_mue(x);
    end

    % UUEs
    idx = idx_uoi(idx_uue);
    uoi_list = tabulate([-1; idx]);
    uoi_list(1,:) = [];
    N_uue = uoi_list(:,2);
    uoi_list = uoi_list(:,1);
    for x = 1:length(uoi_list)
        idx = idx_uoi == uoi_list(x) & idx_uue;
        SE(idx) = log2(1+SIR_uav(idx))/N_uue(x);
    end

    idx_otg = SE < SE_Th1;
    CovPerc = (length(idx_otg)-sum(idx_otg))/length(idx_otg)*100;

    if(PlotCovArea)
        % Plot the coverage layout
        idx_uue = reshape(idx_uue,Res,Res);
        idx_otg = reshape(idx_otg,Res,Res);
        Rmat = Cmue_col(1)*ones(Res,Res);
        Gmat = Cmue_col(2)*ones(Res,Res);
        Bmat = Cmue_col(3)*ones(Res,Res);
        Rmat(idx_uue) = Cuue_col(1); Gmat(idx_uue) = Cuue_col(2); Bmat(idx_uue) = Cuue_col(3);
        Rmat(idx_otg) = Cotg_col(1); Gmat(idx_otg) = Cotg_col(2); Bmat(idx_otg) = Cotg_col(3);
        Cmat(:,:,1) = Rmat; Cmat(:,:,2) = Gmat; Cmat(:,:,3) = Bmat;
        figure();
        image(linspace(-b,b,Res),linspace(-b,b,Res),Cmat);
        hold on;
        plot(Xm,Ym,'dr','MarkerFaceColor','r')
        plot(Xuav,Yuav,'mx','MarkerSize',10,'LineWidth',3);
        [vx,vy] = voronoi(Xm,Ym);
        plot(vx,vy,'k');
        set(gca,'FontSize',14);
        xlim([-b b]);
        xlabel('X-axis distance (Km)','FontSize',14);
        ylabel('Y-axis distance (Km)','FontSize',14);
        title(['SE coverage = ' num2str(CovPerc) '%']);
        legend('MBS','UAV');
        ylim([-b b]);
        hold off;
    end
else
    CovPerc = 0;
end

end