function Main()

tic
% Parameter settings
lambda       = 4;           % MBS density (per km^2)
lambdaUE     = 100;         % UE density (per km^2)
LenSimArea   = 10;          % Length of the simulation area (in km)
P_mac_dBm    = 46;          % Macro Tx power (in dBm)
P_uav_dBm    = 30;          % UAV Tx power (in dBm)
K_dB         = -11;         % Attenuation due to geometrical parameters of antennas (in dB)
NoUAVs       = [16 30 56];  % Total number of available UAVs
DestroySeq   = [200 390];
HeightMBS    = 0.03048; % 100 feet     % Height of the MBS (in km)
HeightUAV    = 0.12192; % 400 feet    % Height of the UAV (in km)
NoIte        = 5;
PLE          = 4;
REB_dB       = 0:3:12;

PlotStyle = {'b', 'r--','m-.','g:','k','c'};

Nm = round(lambda*LenSimArea^2); % Number of MBSs
Nue = round(lambdaUE*(0.7*LenSimArea)^2); % Number of UEs
Pm = 10^(P_mac_dBm/10)/1000 * 10^(K_dB/10); % Net transmit power in Watts
Puav = 10.^(P_uav_dBm/10)/1000* 10^(K_dB/10); % Net transmit power in Watts
REB = 10.^(REB_dB/10);

SE_5percSum_uav = zeros(length(DestroySeq),length(NoUAVs),length(REB));
for k = 1:NoIte
    disp(['Iteration #' num2str(k)]);
    % Initially generate random MBS and UE location coordinates
    % within a given simulation area
    Xm = LenSimArea*(rand(Nm,1)-0.5);
    Ym = LenSimArea*(rand(Nm,1)-0.5);
    Xue = 0.7*LenSimArea*(rand(Nue,1)-0.5);
    Yue = 0.7*LenSimArea*(rand(Nue,1)-0.5);

    % % Plot the MBS locations
    % figure(1);
    % plot(Xm,Ym,'o'); xlabel('X distance (km)'); ylabel('Y distance (km)');

    SE_5perc_uav = zeros(length(DestroySeq),length(NoUAVs),length(REB));

    for x = 1:length(DestroySeq)
        if(x==1)
            NoDestroyMBS = DestroySeq(x);
        else
            NoDestroyMBS = DestroySeq(x)-DestroySeq(x-1);
        end
        idx_destroy = zeros(NoDestroyMBS,1);
        temp = 1:length(Xm);
        for n = 1:NoDestroyMBS
            idx = randi([1,length(temp)]);
            idx_destroy(n) = temp(idx);
            temp(idx) = [];
        end

        % Destroy the marked MBSs
        Xm(idx_destroy) = [];
        Ym(idx_destroy) = [];
        Zm = HeightMBS*ones(size(Xm));

        for y = 1:length(REB)
            % Optimize the UAV location to have maximum 5th percentile SE
            [SE_5perc_uav(x,:,y), Xuav, Yuav] = GA_OptimizeUAVlocation(Xm, Ym, Zm, Xue, Yue, Pm, Puav, NoUAVs, HeightUAV, LenSimArea, PLE, REB(y));
            save(['Data_REBdB_' num2str(REB_dB(y)) '_DstrSeq_' num2str(DestroySeq(x)) '_ite_' num2str(k) '.mat']);
        end
    end

    SE_5percSum_uav = SE_5percSum_uav + SE_5perc_uav;
    
    % 5th percentile SE v/s path-loss exponent
    figure(1);
    for y = 1:length(DestroySeq)
        for n = 1:length(NoUAVs)
            plot(REB_dB, squeeze(SE_5percSum_uav(y,n,:))/k, PlotStyle{n}, 'LineWidth', 2); hold on;
        end
        xlabel('Range expansion bias (in dB)');
        ylabel('5th percentile spectral efficiency (bps/Hz)');
        legend('16 UAVs', '30 UAVs', '56 UAVs', 'location', 'Best');
        pause(100e-3);
    end
    hold off; grid on;
end
toc
end