function Main()

tic
clc;
% Parameter settings
lambda       = 4;           % MBS density (per km^2)
lambdaUE     = 100;         % UE density (per km^2)
LenSimArea   = 10;          % Length of the simulation area (in km)
% SE_Th        = 0.01;        % SE threshold (in bps/Hz)
P_mac_dBm    = 46;          % Macro Tx power (in dBm)
P_uav_dBm    = 30;          % UAV Tx power (in dBm)
K_dB         = -11;         % Attenuation due to geometrical parameters of antennas (in dB)
NoUAVs       = [16 30 56];  % Total number of available UAVs
DestroySeq   = [200 390];
HeightMBS    = 0.03048;     % Height of the MBS (in km)
HeightUAV    = 0.12192;     % Height of the UAV (in km)
% PlotCovArea  = 0;
NoIte        = 10;
PLE          = 4;
REB_dB       = 0:12;

PlotStyle = {'b', 'r--','m-.','g:','k','c'};

Nm = round(lambda*LenSimArea^2); % Number of MBSs
Nue = round(lambdaUE*(0.7*LenSimArea)^2); % Number of UEs
Pm = 10^(P_mac_dBm/10)/1000 * 10^(K_dB/10); % Net transmit power in Watts
Puav = 10.^(P_uav_dBm/10)/1000* 10^(K_dB/10); % Net transmit power in Watts
REB = 10.^(REB_dB/10);

SE_5percSum_uav = zeros(length(DestroySeq),length(NoUAVs),length(REB));
% CovPercSum_uav = zeros(length(DestroySeq),length(NoUAVs),length(REB)); % Store only for 3 values of No. of UAVs: 5, 10, and 15
for k = 1:NoIte
    disp(['Iteration #' num2str(k)]);
    % Initially generate random MBS and UE location coordinates
    % within a given simulation area
    Xm = LenSimArea*(rand(Nm,1)-0.5);
    Ym = LenSimArea*(rand(Nm,1)-0.5);
    Xue = 0.7*LenSimArea*(rand(Nue,1)-0.5);
    Yue = 0.7*LenSimArea*(rand(Nue,1)-0.5);

    % % Plot the MBS locations
    % figure(1);
    % plot(Xm,Ym,'o'); xlabel('X distance (km)'); ylabel('Y distance (km)');

    SE_5perc_uav = zeros(length(DestroySeq),length(NoUAVs),length(REB));
%     CovPerc_uav = zeros(length(DestroySeq),length(NoUAVs),length(REB));

    for x = 1:length(DestroySeq)
        if(x==1)
            NoDestroyMBS = DestroySeq(x);
        else
            NoDestroyMBS = DestroySeq(x)-DestroySeq(x-1);
        end
        idx_destroy = zeros(NoDestroyMBS,1);
        temp = 1:length(Xm);
        for n = 1:NoDestroyMBS
            idx = randi([1,length(temp)]);
            idx_destroy(n) = temp(idx);
            temp(idx) = [];
        end

        % Destroy the marked MBSs
        Xm(idx_destroy) = [];
        Ym(idx_destroy) = [];
        Zm = HeightMBS*ones(size(Xm));

        for y = 1:length(REB)
            for n = 1:length(NoUAVs)
                [Xuav, Yuav] = FindUAVlocation_HexGrid(NoUAVs(n), 0.7*LenSimArea);
                Zuav = HeightUAV*ones(size(Xuav));
                SE_5perc_uav(x,n,y) = Calc5thPercentileSE(Xm, Ym, Zm, Xue, Yue, Pm, PLE, Xuav, Yuav, Zuav, Puav, REB(y));
%                 CovPerc_uav(x,n,y) = CalcCovArea(Xm, Ym, Zm, Pm, PlotCovArea, LenSimArea, PLE, SE_Th, lambdaUE, Xuav, Yuav, Zuav, Puav, REB(y));
            end
        end
    end

%     CovPercSum_uav = CovPercSum_uav + CovPerc_uav;
    SE_5percSum_uav = SE_5percSum_uav + SE_5perc_uav;
    
    save('Data1.mat');
    
    % 5th percentile SE v/s path-loss exponent
    figure(1);
    for x = 1:length(DestroySeq)
        for n = 1:length(NoUAVs)
            plot(REB_dB, squeeze(SE_5percSum_uav(x,n,:))/NoIte,PlotStyle{n}, 'LineWidth', 2); hold on;
        end
    end
    xlabel('Range expansion bias (in dB)');
    ylabel('5th percentile spectral efficiency (bps/Hz)');
    legend('16 UAVs', '30 UAVs', '56 UAVs', 'location', 'Best');
    hold off; grid on;
    pause(100e-3);
end
toc
end